searchData={"items":[{"type":"module","title":"DoublyLinkedList","doc":"A fast, ammortised O(log n) doubly linked list implementation.\n\nA doubly linked list is a type of linked list in which each node contains\nthree elements: a data value, a pointer to the next node in the list, and a\npointer to the previous node. This two-way linkage allows traversal of the\nlist in both directions, forward and backward, which is a significant\nadvantage over a singly linked list that can only be traversed in one\ndirection.\n\nA new doubly linked list can be constructed using `new/0`:\n\n```\niex> dll = DoublyLinkedList.new()\n#DoublyLinkedList \n```\n\nMany types of insertion and deletion are supported, the most basic being\n`insert_head/2`, `insert_tail/2`, `remove_head/1` and `remove_tail/2` which\nwill insert and remove at the head and tail of the doubly linked list\nrespectively:\n\n```\niex> dll = DoublyLinkedList.new()\n#DoublyLinkedList \n\niex> {dll, node} = DoublyLinkedList.insert_tail(dll, \"tail value\")\n{#DoublyLinkedList , #DoublyLinkedNode }\n\niex> {dll, node} = DoublyLinkedList.insert_head(dll, \"head value\")\n{#DoublyLinkedList , #DoublyLinkedNode }\n\niex> dll = DoublyLinkedList.remove_tail(dll)\n#DoublyLinkedList \n\niex> dll = DoublyLinkedList.remove_head(dll)\n#DoublyLinkedList \n```\n\nNode data can be updated with `update/3`:\n\n```\niex> dll = DoublyLinkedList.new()\n#DoublyLinkedList \n\niex> {dll, node} = DoublyLinkedList.insert_tail(dll, \"tail value\")\n{#DoublyLinkedList , #DoublyLinkedNode }\n\niex> dll = DoublyLinkedList.update(dll, node, \"new tail value\")\n#DoublyLinkedList \n```\n\nThere are two options for traversal. `DoublyLinkedList` implements the\nEnumerable protocol so `Enum.map/2`, `Enum.member?/2`,\n[etc](https://hexdocs.pm/elixir/1.12/Enum.html#functions) are supported; there\nare also the built-in `find_from_head/2` and `find_from_tail/2` functions.\n`find_from_tail/2` is of particular interest as it traverses the list in\nreverse. Note that traversal methods are O(n).\n\n```\niex> dll = DoublyLinkedList.new()\niex> {dll, node} = DoublyLinkedList.insert_tail(dll, 1)\niex> {dll, node} = DoublyLinkedList.insert_tail(dll, 2)\n{#DoublyLinkedList , #DoublyLinkedNode }\n\niex> Enum.map(dll, fn v -> v * 2 end)\n[2, 4]\n```","ref":"DoublyLinkedList.html"},{"type":"function","title":"DoublyLinkedList.find_from_head/2","doc":"Find the first node whose data matches the given value (starting from the head\nof the list).","ref":"DoublyLinkedList.html#find_from_head/2"},{"type":"function","title":"DoublyLinkedList.find_from_tail/2","doc":"Find the first node whose data matches the given value (starting from the tail\nof the list).","ref":"DoublyLinkedList.html#find_from_tail/2"},{"type":"function","title":"DoublyLinkedList.get/2","doc":"Get the node.","ref":"DoublyLinkedList.html#get/2"},{"type":"function","title":"DoublyLinkedList.get_head/1","doc":"Get the node at the head of the list.","ref":"DoublyLinkedList.html#get_head/1"},{"type":"function","title":"DoublyLinkedList.get_next/2","doc":"Get the node after the given node.","ref":"DoublyLinkedList.html#get_next/2"},{"type":"function","title":"DoublyLinkedList.get_prev/2","doc":"Get the node before the given node.","ref":"DoublyLinkedList.html#get_prev/2"},{"type":"function","title":"DoublyLinkedList.get_tail/1","doc":"Get the node at the tail of the list.","ref":"DoublyLinkedList.html#get_tail/1"},{"type":"function","title":"DoublyLinkedList.insert_after/3","doc":"Insert a node after the given node.","ref":"DoublyLinkedList.html#insert_after/3"},{"type":"function","title":"DoublyLinkedList.insert_before/3","doc":"Insert a node before the given node.","ref":"DoublyLinkedList.html#insert_before/3"},{"type":"function","title":"DoublyLinkedList.insert_head/2","doc":"Insert a node at the head of the list.","ref":"DoublyLinkedList.html#insert_head/2"},{"type":"function","title":"DoublyLinkedList.insert_tail/2","doc":"Insert a node at the tail of the list.","ref":"DoublyLinkedList.html#insert_tail/2"},{"type":"function","title":"DoublyLinkedList.new/0","doc":"Construct a new list.","ref":"DoublyLinkedList.html#new/0"},{"type":"function","title":"DoublyLinkedList.remove/2","doc":"Remove the node.","ref":"DoublyLinkedList.html#remove/2"},{"type":"function","title":"DoublyLinkedList.remove_after/2","doc":"Remove the node after the given node.","ref":"DoublyLinkedList.html#remove_after/2"},{"type":"function","title":"DoublyLinkedList.remove_before/2","doc":"Remove the node before the given node.","ref":"DoublyLinkedList.html#remove_before/2"},{"type":"function","title":"DoublyLinkedList.remove_head/1","doc":"Remove the node at the head of the list.","ref":"DoublyLinkedList.html#remove_head/1"},{"type":"function","title":"DoublyLinkedList.remove_tail/1","doc":"Remove the node at the tail of the list.","ref":"DoublyLinkedList.html#remove_tail/1"},{"type":"function","title":"DoublyLinkedList.update/3","doc":"Update the node with the given value.","ref":"DoublyLinkedList.html#update/3"},{"type":"type","title":"DoublyLinkedList.t/0","doc":"","ref":"DoublyLinkedList.html#t:t/0"},{"type":"module","title":"DoublyLinkedList.Node","doc":"A doubly linked list node.","ref":"DoublyLinkedList.Node.html"},{"type":"function","title":"DoublyLinkedList.Node.new/2","doc":"","ref":"DoublyLinkedList.Node.html#new/2"},{"type":"type","title":"DoublyLinkedList.Node.t/0","doc":"","ref":"DoublyLinkedList.Node.html#t:t/0"}],"content_type":"text/markdown"}